function [X,INFOS] = PenCorr_lub_HnormMajorDiag(G,H,ConstrA,Rank,OPTIONS)
%%%%%%%%%%%%% This code is designed to solve %%%%%%%%%%%%%%%%%%%%%
%%       min   0.5*|| H o (X-G) ||^2
%%       s.t.  X_ii   = e_ij    (i,j) in (I_e,J_e)
%%             X_ij  >= l_ij    (i,j) in (I_l,J_l)
%%             X_ij  <= u_ij    (i,j) in (I_u,J_u)
%%            rank(X) <= Rank     
%%              X    >= tau*I    X is SDP (tau>=0 and may be zero)
%%%
%   Parameters:
%   Input
%   G:            the given symmetric correlation matrix
%   H:            the weight matrix for G
%   ConstrA:      the equality and inequality constraints
%   Rank:         the rank constraint to X
%   OPTIONS: 
%
%   Output
%   X         the optimal primal solution
%   INFOS     the final information
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Last modified on June 22, 2010.


%%
%%-----------------------------------------
%%% get constraints infos from constrA
%%-----------------------------------------
%%
e   = ConstrA.e; I_e = ConstrA.Ie; J_e = ConstrA.Je;
l   = ConstrA.l; I_l = ConstrA.Il; J_l = ConstrA.Jl;
u   = ConstrA.u; I_u = ConstrA.Iu; J_u = ConstrA.Ju;
k_e = length(e); k_l = length(l);  k_u = length(u);
k   = k_e + k_l + k_u;  n = length(G);


tau         = 0;
maxit       = 500; 
maxitsub    = 100;
tolrel      = 1.0e-5;   % default
tolrank     = 1.0e-8;   % no need to change
tolKKT      = 1.0e-5;
scale_data     = 0;     % scaling data
use_AlterProj  = 0;     % =1 initial point from an alternating projection method 
use_AlterMajor = 0;     % =1 initial point from an alternative majorization method
switch_restart = 0;  
name_itsolver  = 'BiCGStab';  % or 'IDRS'
%%                                      
%%-----------------------------------------
%% get parameters from the OPTIONS structure. 
%%-----------------------------------------
%%
if exist('OPTIONS')
    if isfield(OPTIONS,'tau');                tau               = OPTIONS.tau; end
    if isfield(OPTIONS,'tolrel');             tolrel            = OPTIONS.tolrel; end
    if isfield(OPTIONS,'tolrank');            tolrank           = OPTIONS.tolrank; end
    if isfield(OPTIONS,'maxit');              maxit             = OPTIONS.maxit; end
    if isfield(OPTIONS,'maxitsub');           maxitsub          = OPTIONS.maxitsub; end
    if isfield(OPTIONS,'scale_data');         scale_data        = OPTIONS.scale_data; end
    if isfield(OPTIONS,'use_AlterProj');      use_AlterProj     = OPTIONS.use_AlterProj ; end
    if isfield(OPTIONS,'use_AlterMajor');     use_AlterMajor    = OPTIONS.use_AlterMajor; end
    if isfield(OPTIONS,'switch_restart');     switch_restart    = OPTIONS.switch_restart; end
    if isfield(OPTIONS,'name_itsolver');      name_itsolver     = OPTIONS.name_itsolver; end
end
innerTolrel = tolrel;
tolsub      = max(innerTolrel, 1.0*tolrel);  
tolsub_rank = tolsub; 

finalPCA = 0;
residue_cutoff  = 10;
if tolrel <= 1.0e-4;
   residue_cutoff = 100;
end
%%% constant pars
const.disp1 = 10;
const.disp2 = 10;
const.rank_hist    = 10; %5;  const.rank_hist    = max(2, const.rank_hist);  
const.rankErr_hist = 10; %3;  const.rankErr_hist = max(2, const.rankErr_hist);  
const.funcVal_hist = 2;  %10; 4;  const.funcVal_hist = max(2, const.funcVal_hist);
const.residue_hist = const.funcVal_hist;  const.residue_hist = max(2, const.residue_hist); 
runhist.rank    = zeros(const.rank_hist,1);
runhist.rankErr = zeros(const.rankErr_hist,1);
runhist.funcVal = zeros(const.funcVal_hist,1);
runhist.residue = zeros(const.residue_hist,1);
progress_rank1    = 1;
progress_rank2    = 1;
%progress_residue  = 1.0e-4
progress_relErr   = 0.001*tolrel;
progress_rankErr  = 1.0e-3;
Totalcall_CaliMatW   = 0;
Totaliter_CaliMatW   = 0;
Totalnumb_JacobProd  = 0;
Totalnumb_eigendecom = 0;
%%% penalty par
c0_min = 1.0;
c0_max = 1e3;  %1e2;   
alpha_min  = 1.2; %1.4
alpha_max  = 4;
%alpha_max2 = 10;
c_max      = 1.0e8;


t0 = clock;

%%% reset input pars
eye_n  = speye(n);
G      = G - tau*eye_n;     
G      = (G + G')/2;        
Idx    = find(I_e==J_e);
e(Idx) = e(Idx) - tau;     % reset the diagonal part of e
e_diag = e(Idx);
ConstrA.e = e;


%%% scale H
H = (H + H')/2;    % make sure that H is symmetric
if ( scale_data )
    Hscale = sum(sum(H))/n^2;
    H      = H/Hscale;
end
H2H = H.*H;
% H_max  = max(max(H));
% H_max2 = H_max^2;   


%%% check if H=ones(n)
weightFlag = any(any(H-ones(n)));  % =0 if equal weights
if weightFlag == 0
    msg = sprintf('Equal weight case: call PenCorr_lub.m');
    fprintf('\n %s ', msg);
    OPTIONS.tolrel = 1.0e-5;
    [X,INFOS] = PenCorr_lub(G,ConstrA,Rank,OPTIONS);
    return;
end

%% to control the overestimation about w
w_scalar = 1.4;  
d        = diag(H);
if isfield(OPTIONS,'majorW')
    W      = OPTIONS.majorW;
    w_est0 = diag(W);
else
    w_est0 = max(H,[],2);
end
%% note that d<= w_est should be guaranteed
w_diff = max(0,w_est0-d) + 1.0e-6;    % e-6 to avoid numerical instability
Delta  = max(0, H2H - d*d');
DW = d*w_diff' + w_diff*d';
DD = -DW + ( DW.^2 + 4*(Delta.*(w_diff*w_diff')) ).^(1/2);
DD = 0.5*(DD./(w_diff*w_diff'));
alpha = max(max(DD));
%fprintf('\n alpha = %d',alpha)
%%%%% alpha is chosen such that [d+alpha*(w-d)_+]*[d+alpha*(w-d)_+]^T >= H2H
% if alpha <= 0.1
%     w_est = d + alpha*w_est0;
% end
w_est = d + alpha*(w_diff - 1.0e-6);

w     = w_est/w_scalar;
Ind   = find(abs(w)<=1.0e-12);
w(Ind) = 1;
% w_min = min(w);
% w_max = max(w);

W = diag(w);
w_inv = w.^(-1);

% w_min = min(w);
% w_max = max(w);
    
    
    
% %%% form W
% if isfield(OPTIONS,'majorW')
%     W = OPTIONS.majorW;
%     w = diag(W);
%     w_inv = w.^(-1);
% else
% %     %%% put weigths 0 for the fixed elements
% %     for i=1:k_e
% %         H(I_e(i),J_e(i)) = 0;
% %         H(J_e(i),I_e(i)) = 0;
% %     end  
%     w = max(H,[],2); 
%     w_min = min(w);
%     w_max = max(w);
%     if (w_max - w_min)/w_max <= 0.25
%         w = w_max*ones(n,1);
%     end    
%     Ind = find(w==0);
%     w(Ind) = 1;
%     
%     w = w/1.4;  
%     w_inv = w.^(-1);
%     W = diag(w);
% end



fprintf('\n ******************************************')
fprintf('\n  The problem information: \n')
fprintf(' Dim.  of sdp  constr.    = %3.0f \n',n)
fprintf(' Num. of fixed elements   = %3.0f \n',k_e)
fprintf(' Num. of lower bound      = %3.0f \n',k_l)
fprintf(' Num. of upper bound      = %3.0f \n',k_u)
fprintf(' The required rank of X   = %3.0f \n',Rank)
%%
%%---------------------------------------------------------------
%%% CorNewton3Mex preprocessing
%%---------------------------------------------------------------
%%
fprintf('\n ^^^^^^^^ Preprocessing by CaliMat1DiagMex ^^^^^^^^ ')
opts.eps_bar     = 1.0e1;
optsInt.disp     = 1;
optsInt.finalEig = 1;
optsInt.name_itsolver = name_itsolver;
%[X,z,infoInt] = CaliMat1Mex_Wnorm(G,W,ConstrA,optsInt); 
[X,z,infoInt] = CaliMat1DiagMex(G,W,ConstrA,optsInt); %June 10, 2010
P          = infoInt.P;
lambda     = infoInt.lam;
rank_X     = infoInt.rank;
Totalcall_CaliMatW   = Totalcall_CaliMatW + 1;
Totaliter_CaliMatW   = Totaliter_CaliMatW + infoInt.numIter;
Totalnumb_JacobProd  = Totalnumb_JacobProd + infoInt.numJacobProd;
Totalnumb_eigendecom = Totalnumb_eigendecom + infoInt.numEig;
objM = H.*(X-G);
residue_CaliMatW = sum(sum(objM.*objM))^0.5;
rankErr_CaliMatW = abs( sum(lambda) - sum(lambda(1:Rank)) );
if  ( rankErr_CaliMatW <= tolrank )    
    % check KKT condition
    KKT = H2H - w*w'; %ones(n,n); % H2H - w*w' if use w;
    KKT = KKT.*(X-G);
    Err_KKT = sum(sum(KKT.*KKT))^0.5;
    Err_KKT = Err_KKT/max(w)^2;
    if ( Err_KKT <= tolKKT )  
    fprintf('\n The initial calibration is good enough!')
    fprintf('\n The rank error    = %4.3e', rankErr_CaliMatW)
    fprintf('\n The KKT error     = %4.3e', Err_KKT)
    fprintf('\n Residue_CaliMatW  = %9.8e', residue_CaliMatW)
    time_used = etime(clock,t0);
    fprintf('\n Total computing time   = %.1f(secs) \n', time_used);
    if Rank<n
        fprintf('\n lambda(r) - lambda(r+1) === %d \n',lambda(Rank)-lambda(Rank+1));
    end    
    INFOS.iter         = 0;
    INFOS.callCN       = Totalcall_CaliMatW;
    INFOS.itCN         = Totaliter_CaliMatW;
    INFOS.itJacobProd  = Totalnumb_JacobProd;
    INFOS.numEig  = Totalnumb_eigendecom;
    INFOS.rank    = rank_X;
    INFOS.rankErr = rankErr_CaliMatW;
    INFOS.residue = residue_CaliMatW;
    INFOS.time    = time_used;
    return;
    end
end
fprintf('\n The rank of CaliMatW     = %3.0d', rank_X)
fprintf('\n The rankErr of CaliMatW  = %4.3e', rankErr_CaliMatW)  
fprintf('\n Residue_CaliMatW         = %9.8e', residue_CaliMatW)

%%% reset rank
if switch_restart == 1
    rank_step = 10;
    Rank0 = Rank;
    Rank  = round(n/rank_step);
    Rank  = max(Rank0,Rank);
end


%% Initial point
fprintf('\n\n ^^^^^^^^ Initial Guess ^^^^^^^^ ')
if use_AlterProj == 1
    fprintf('\n use_AlterProj!')
    opt_disp = 1;
    [X,infos_AlterProj] = IntPoint_Hnormlub(G,H,W,ConstrA,Rank,X,P,lambda,opt_disp);    
    %[X,P,lambda,rank_X,rankErr,infoNum] = InitialPoint(G,H_max2,ConstrA,Rank,rankErr_CaliMatW,X1); 
    
    Totalcall_CaliMatW   = Totalcall_CaliMatW + infos_AlterProj.callCN;
    Totaliter_CaliMatW   = Totaliter_CaliMatW + infos_AlterProj.iterCN;
    Totalnumb_JacobProd  = Totalnumb_JacobProd + infos_AlterProj.JacobProd;
    Totalnumb_eigendecom = Totalnumb_eigendecom + infos_AlterProj.eigendecom;  
    
    P       = infos_AlterProj.P;
    lambda  = infos_AlterProj.lam;
    rank_X  = infos_AlterProj.rank;
    %rankErr = infos_AlterProj.rankErr;
    residue_int = infos_AlterProj.residue;
%     objM = H.*(X-G);
%     residue_int = sum(sum(objM.*objM))^0.5;    
    residue_1   = residue_int;
    time_int    = infos_AlterProj.time;
    fprintf('    Time used = %.1f', time_int);
elseif use_AlterMajor == 1
    fprintf('\n use_AlterMajor!')
    opts_AlterMajor.tolrel = 1.0e-6;
    opts_AlterMajor.tolInf = 1.0e-5;
    opts_AlterMajor.disp   = 1;
    [X,infos_AlterMajor]   = PenCorrlubHnormMajor_New5(G,H,ConstrA,Rank,opts_AlterMajor);
    [P,lambda] = MYmexeig(X);
    rank_X      = Rank;
    residue_int = infos_AlterMajor.residue;
    time_int    = infos_AlterMajor.time;
    fprintf('    Time used = %.1f', time_int);
%     objM = H.*(X-G);
%     residue_int = sum(sum(objM.*objM))^0.5;    
    residue_1   = residue_int;
else
    X = mPCA(P,lambda,Rank,e_diag);
    [P,lambda] = MYmexeig(X);
    Totalnumb_eigendecom = Totalnumb_eigendecom + 1;
    objM = H.*(X-G);
    residue_CaliMatWPCA = sum(sum(objM.*objM))^0.5;
    rank_X      = Rank;
    residue_int = residue_CaliMatWPCA;
    residue_1   = residue_int;
end 
%%% initialize U
P1 = P(:,1:Rank);
U  = P1*P1';
rankErr = abs( sum(lambda) - sum(lambda(1:Rank)) );


%%
%%---------------------------------------------------------------
%%% initial penalty parameter c
%%---------------------------------------------------------------
%%
if use_AlterProj 
    c0 = 0.50*(residue_int^2 - residue_CaliMatW^2);
    c0 = 0.25*c0/max(1.0, rankErr_CaliMatW - rankErr);
elseif use_AlterMajor 
    c0 = 0.50*(residue_int^2 - residue_CaliMatW^2);
    c0 = 0.25*c0/max(1.0, rankErr_CaliMatW);
    c0 = 14*c0;
else  %use_CaliMatWPCA
    c0 = 0.50*(residue_CaliMatWPCA^2 - residue_CaliMatW^2);
    c0 = 0.25*c0/max(1.0, rankErr_CaliMatW); 
end
if Rank <= 1
    c0 =  c0_max ;
end


if tolrel >= 1.0e-1;  %% less acurate, larger c
    c0 = 4*c0;
elseif tolrel >= 1.0e-2;  %% less acurate, larger c
    c0 = 2*c0;
% elseif tolrel >= 1.0e-3;  %% less acurate, larger c
%     c0 = 2*c0;
% elseif tolrel >= 1.0e-4;  %% less acurate, larger c
%     c0 = 2*c0;
end
% if tolrel <= 1.0e-6;  %% more acurate, smaller c
%     c0 = c0/2;
% end


c0 = max(c0, c0_min);
c0 = min(c0, c0_max);
c  = c0;


%% main 
fprintf('\n\n ************************************** \n')
fprintf( '      The Penalty Method Initiated!!!      ')
fprintf('\n ************************************** \n')
fprintf('The initial rank          = %3.0d \n', rank_X);
fprintf('The initial rank error    = %4.3e \n', rankErr);
fprintf('The initial ||Ho(X0-G)||  = %9.8e \n', residue_1);

relErr_0     = 1.0e6;
break_level  = 0;
eps_bar_min  = 1.0e-4;
opts.eps_bar = 1.0e1;

k1       = 1;
sum_iter = 0; 
opts.disp     = 0;
opts.finalEig = 1;
while ( k1 <= maxit )
    
    subtotaliter_CaliMatW   = 0;
    subtotalnumb_JacobProd  = 0;
    subtotalnumb_eigendecom = 0;
            
    fc = 0.5*residue_1^2;
    fc = fc + c*rankErr;
    fc_old = fc;
    
    tt = etime(clock,t0);
    fprintf('\n =============')
    fprintf(' The %2.0dth level of penalty par. %6.5e',k1,c)
    fprintf('  =====================')
    fprintf('\n ........Calling CaliMat1Mex_Wnorm')
    fprintf('\n CallNo.  NumIt  JacProd    RankX    RankErr      Sqrt(2*FunVal)     Time')
    fprintf('\n %2.0fth      %s       %s        %3.0d      %3.2e     %9.8e    %.1f',...
        0,'-','-',rank_X,rankErr,sqrt(2)*fc^0.5,tt)
    
    C = H2H.*(X - G) + c*(eye_n - U);
    for i=1:n
        C(i,:) = w_inv(i)*C(i,:);
    end
    for j=1:n
        C(:,j) = C(:,j)*w_inv(j);
    end
    G0  = X - C;
    
    pre_number = 0;  % the number of iterations for choosing equal weights
    for itersub = 1:maxitsub
         opts.name_itsolver = name_itsolver;
         if ( k1==1 && itersub==1  )
            if k1 <= pre_number
                w_max = max(w);
                W_pre = w_max*eye_n;
                fprintf('use W_pre!')
                %[X,z,info] = CaliMat1Mex_Wnorm(G0,W_pre,ConstrA,opts);
                [X,z,info] = CaliMat1DiagMex(G0,W_pre,ConstrA,opts);
            else
                opts.smstep_help = 1;                
                opts.tolsmstep   = 1.0e-4;
                %[X,z,info] = CaliMat1Mex_Wnorm(G0,W,ConstrA,opts);
                [X,z,info] = CaliMat1DiagMex(G0,W,ConstrA,opts);                
            end
            
        else
            if k1 <= pre_number
                w_max = max(w);
                W_pre = w_max*eye_n;
                fprintf('use W_pre!')
                [X,z,info] = CaliMat1DiagMex(G0,W_pre,ConstrA,opts,z);
            else
                opts.smstep_help = 1;
                opts.tolsmstep = 1.0e-4;
                [X,z,info] = CaliMat1DiagMex(G0,W,ConstrA,opts,z);
            end
         end         
%         if ( itersub==1 && rankErr > tolrank )
%             if k1 <= pre_number
%                 w_max = max(w);
%                 W_pre = w_max*eye_n;
% 
%                 %[X,z,info] = CaliMat1Mex_Wnorm(G0,W_pre,ConstrA,opts);
%                 [X,z,info] = CaliMat1DiagMex(G0,W_pre,ConstrA,opts);
%             else
%                 
%                 %[X,z,info] = CaliMat1Mex_Wnorm(G0,W,ConstrA,opts);
%                 [X,z,info] = CaliMat1DiagMex(G0,W,ConstrA,opts);
%                 
%             end
%             
%         else
%             if k1 <= pre_number
%                 w_max = max(w);
%                 W_pre = w_max*eye_n;
%  
%                 [X,z,info] = CaliMat1DiagMex(G0,W_pre,ConstrA,opts,z);
%             else
%                 [X,z,info] = CaliMat1DiagMex(G0,W,ConstrA,opts,z);
%             end
%         end
        P          = info.P;
        lambda     = info.lam;
        rank_X     = info.rank;
        rankErr    = abs( sum(lambda) - sum(lambda(1:Rank)) ); 
        Totalcall_CaliMatW      = Totalcall_CaliMatW + 1;
        subtotaliter_CaliMatW   = subtotaliter_CaliMatW + info.numIter;
        subtotalnumb_JacobProd  = subtotalnumb_JacobProd+ info.numJacobProd;       
        subtotalnumb_eigendecom = subtotalnumb_eigendecom + info.numEig;               
        objM = H.*(X-G);        
        fc   = sum(sum(objM.*objM));        
        residue_1 = fc^0.5;      
        fc        = 0.5*fc + c*rankErr;
       
        if ( itersub <= const.disp1 || mod(itersub,const.disp2) == 0 )
            dispsub = 1;
            tt = etime(clock,t0);
            fprintf('\n %2.0dth     %2.0d      %3.0d       %3.0d      %3.2e     %9.8e    %.1f',...
                itersub, info.numIter, info.numJacobProd, rank_X, rankErr, sqrt(2)*fc^0.5, tt)
        else
            dispsub = 0;
        end
 
        %%% function value history
        if  itersub <= const.funcVal_hist
            runhist.funcVal(itersub) = fc^0.5;
        else
            for j = 1:const.funcVal_hist-1
                runhist.funcVal(j) = runhist.funcVal(j+1);
            end
            runhist.funcVal(const.funcVal_hist) = fc^0.5;
        end
        %%% residue history
        if sum_iter + itersub <= const.residue_hist
            runhist.residue(sum_iter + itersub) = residue_1;
        else
            for j = 1:const.residue_hist-1
                runhist.residue(j) = runhist.residue(j+1);
            end
            runhist.residue(const.residue_hist) = residue_1;
        end
        %%% rank history
        if  itersub <= const.rank_hist
            runhist.rank(itersub) = rank_X;
        else
            for j = 1:const.rank_hist-1
                runhist.rank(j) = runhist.rank(j+1);
            end
            runhist.rank(const.rank_hist) = rank_X;
        end

        if fc > fc_old
            w_scalar = w_scalar/1.1;
            w_scalar = max(1,w_scalar);
            if w_scalar > 1
                w     = w_est/w_scalar;
                w_min = min(w);
                w_max = max(w);
                if (w_max-w_min)/w_max <= 0.25
                    w = w_max*ones(n,1);
                end
                Ind    = find(abs(w)<=1.0e-12);
                w(Ind) = 1;

                W     = diag(w);
                w_inv = w.^(-1);
            end
        end
        fc_old = fc;
        
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        if switch_restart == 1 
            if (Rank ~= Rank0 && rankErr <= tolrank)
            fprintf('\n Rank %2.0d is attained and restart the penalty algorithm!',Rank)
            %Rank = round(Rank/rank_step);
            %Rank = max(Rank0,Rank);
            Rank = Rank0;
            rankErr = 1.0e+6;  %%% put a large number to restart            
            break;
            end
        end
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
        if rankErr <= tolrank
            tolsub_check = tolsub_rank;       
        else
            tolsub_check = tolsub*max(10, min(100,rank_X/Rank));
        end
        if itersub >= const.funcVal_hist
           relErr_sub = abs(runhist.funcVal(1) - runhist.funcVal(const.funcVal_hist));
           relErr_sub = relErr_sub/max(residue_cutoff, max(runhist.funcVal(1), runhist.funcVal(const.funcVal_hist)));
        end
        if ( itersub >= const.funcVal_hist &&  relErr_sub <= tolsub_check )
            if dispsub == 0
                tt = etime(clock,t0);
                fprintf('\n %2.0dth     %2.0d     %3.0d        %3.0d        %3.2e   %9.8e    %.1f',...
                    itersub, info.numIter, info.numJacobProd, rank_X, rankErr, sqrt(2)*fc^0.5, tt)
            end
            break;
        elseif ( itersub >= const.rank_hist && abs( runhist.rank(1) - runhist.rank(const.rank_hist) ) <= progress_rank1...
                && rank_X - Rank >= progress_rank2 )
            %fprintf('\n Warning: The rank does not decrease in this level!')
            if dispsub == 0
                tt = etime(clock,t0);
                fprintf('\n %2.0dth     %2.0d     %3.0d        %3.0d        %3.2e   %9.8e    %.1f',...
                    itersub, info.numIter, info.numJacobProd, rank_X, rankErr, sqrt(2)*fc^0.5, tt)
            end
            break;
        end
               
        %%% update U, G0 and fc0
        P1 = P(:, 1:Rank);
        U  = P1*P1';
        
        C = H2H.*(X - G) + c*(eye_n - U);
        for j=1:n
            C(j,:) = w_inv(j)*C(j,:);
        end
        for j=1:n
            C(:,j) =  C(:,j)*w_inv(j);
        end
        G0  = X - C;
        
        if itersub <= const.residue_hist
            opts.eps_bar = opts.eps_bar/(10^itersub);
            opts.eps_bar =  max( opts.eps_bar, eps_bar_min);
        else
            opts.eps_bar = min(opts.eps_bar, min(1,relErr_sub)^2);
            opts.eps_bar =  max( opts.eps_bar, eps_bar_min);
        end
        
    end   %end of subproblem
    
    sum_iter             = sum_iter + itersub;     
    Totalnumb_JacobProd  = Totalnumb_JacobProd + subtotalnumb_JacobProd;
    Totaliter_CaliMatW   = Totaliter_CaliMatW + subtotaliter_CaliMatW;
    Totalnumb_eigendecom = Totalnumb_eigendecom + subtotalnumb_eigendecom;
    fprintf('\n SubTotal %2.0f      %3.0f      %2.0f(eigendecom)',...
        subtotaliter_CaliMatW,subtotalnumb_JacobProd,subtotalnumb_eigendecom);
    
    if sum_iter >= const.residue_hist
        relErr = abs(runhist.residue(1) - runhist.residue(const.residue_hist));
        relErr = relErr/max(residue_cutoff, max(runhist.residue(1), runhist.residue(const.residue_hist)));
    else
        relErr = abs(runhist.residue(1) - runhist.residue(sum_iter));
        relErr = relErr/max(residue_cutoff, max(runhist.residue(1), runhist.residue(sum_iter)));
    end   
    tt = etime(clock,t0);
    [hh,mm,ss] = time(tt);
    fprintf('\n   Iter.   PenPar.   Rank(X)    RankError     relErr     ||Ho(X-G)||     Time_used ')
    fprintf('\n   %2.0d     %3.2e    %2.0d      %3.2e     %3.2e  %9.8e   %d:%d:%d \n',...
        k1,c,rank_X,rankErr,relErr,residue_1,hh,mm,ss)
   
    
    if  k1 <= const.rankErr_hist
        runhist.rankErr(k1) = rankErr;
    else
        for j = 1:const.rankErr_hist-1
            runhist.rankErr(j) = runhist.rankErr(j+1);
        end
        runhist.rankErr(const.rankErr_hist) = rankErr;
    end   
    %%% termination test
    if ( relErr <= tolrel )
        if ( rankErr <= tolrank )
            fprintf('\n The rank constraint is satisfied!')
            break;
        elseif ( k1 >= const.rankErr_hist && abs(runhist.rankErr(1) - runhist.rankErr(const.rankErr_hist)) <= progress_rankErr )
            fprintf('\n Warning: The rank does not decrease any more! :( ')
            finalPCA = 1;
            break;
        end
    else
        %         if ( abs(residue_0 - residue_1)/max(1,max(residue_0,residue_1)) <= progress_residue )
        %             fprintf('\n Warning: The residue is decreasing slowly, quit! :( ')
        %             if ( rankErr > tolrank )
        %                 finalPCA = 1;
        %             end
        %             break;
        %         end
        if ( abs(relErr_0 - relErr)/max(1,relErr) <= progress_relErr )
            break_level = break_level + 1;
            if break_level == 3
                fprintf('\n Warning: The relErr is consecutively decreasing slowly, quit! :( ')
                if ( rankErr > tolrank )
                    finalPCA = 1;
                end
                break;
            end
        end       
    end
        
    k1        = k1 + 1; 
    relErr_0  = relErr;
    
    %%% update c
    if switch_restart == 1
        c = min(c_max, c*10);
    else
        if rank_X <= Rank
            c = min(c_max, c);
            fprintf('\n The rank constraint is satisfied and keep c the same!')
        else
            if rankErr/min(10,Rank) > 1.0e-1
                c = min(c_max, c*alpha_max);
            else
                c = min(c_max, c*alpha_min);
            end
        end
    end
    
    %%% update eps_bar for next level penalty
    if rank_X <= Rank
        opts.eps_bar = max( opts.eps_bar, eps_bar_min );
    else
        if k1 <= 3         %const.residue_hist
            opts.eps_bar = opts.eps_bar/(10^k1);
            opts.eps_bar = max( opts.eps_bar, eps_bar_min );
        else
            opts.eps_bar = min(opts.eps_bar, min(1, relErr)^2);
            opts.eps_bar = max( opts.eps_bar, eps_bar_min );
        end
    end
    
end

%% final PCA correction
if finalPCA
    X = mPCA(P,lambda,Rank,e_diag);
    %[P,lambda] = MYmexeig(X);
    %Totalnumb_eigendecom = Totalnumb_eigendecom + 1;
    rank_X  = Rank;
    rankErr = 0;
    objM = H.*(X-G);
    residue_1 = sum(sum(objM.*objM))^0.5;
end
infeas = zeros(k,1);
for i=1:k_e
    infeas(i) = e(i) - X(I_e(i),J_e(i));
end
for i=1:k_l
    infeas(k_e+i) = max( l(i) - X(I_l(i),J_l(i)), 0 );
end
for i=1:k_u
    infeas(k_e+k_l+i) = max( X(I_u(i),J_u(i)) - u(i), 0 );
end
NormInf = norm(infeas);


time_used = etime(clock,t0);
INFOS.iter         = k1;
INFOS.callCN       = Totalcall_CaliMatW;
INFOS.itCN         = Totaliter_CaliMatW;
INFOS.itJacobProd  = Totalnumb_JacobProd;
INFOS.numEig       = Totalnumb_eigendecom;
INFOS.rank    = rank_X;
INFOS.rankErr = rankErr;
INFOS.relErr  = relErr;
INFOS.infeas  = NormInf;
INFOS.residue = residue_1;
INFOS.time    = time_used;
fprintf('\n Final ||Ho(X-G)||      ===== %9.8e', INFOS.residue);
fprintf('\n Primal function value  ===== %9.8e', 0.5*INFOS.residue^2);
fprintf('\n Computing time         ======= %.1f(secs)\n', INFOS.time);

fid = fopen('result_PenCorr_lub_HnormMajorDiag.txt','a+');
fprintf(fid,'\n ***** result_PenCorr_lub_HnormMajorDiag **************');
fprintf(fid,'\n  The problem information: \n');
fprintf(fid,' Dim.  of    sdp        constr  = %d \n',n);
fprintf(fid,' Num. of fixed elements         = %d \n',k_e);
fprintf(fid,' Num. of lower bound            = %d \n',k_l);
fprintf(fid,' Num. of upper bound            = %d \n',k_u);
fprintf(fid,' The required rank      constr  = %d \n',Rank);
fprintf(fid,' ------------------------------------------------------- \n');
%fprintf(fid,'\n **************** Final Information ******************** \n');
fprintf(fid,' Num of pen level         = %d \n', INFOS.iter);
fprintf(fid,' Num of calling CN        = %d \n', INFOS.callCN);
fprintf(fid,' Total num of JacobProda  = %d \n', INFOS.itJacobProd);
fprintf(fid,' Total num of eigendecom  = %d \n', INFOS.numEig);
%fprintf(fid,' Total num of iter in CN  = %d \n', INFOS.itCN);
fprintf(fid,' The rank of  X*          === %d \n', INFOS.rank);
fprintf(fid,' The rank error           === %3.2e \n', INFOS.rankErr);
fprintf(fid,' The rel func error       === %3.2e \n', INFOS.relErr);
fprintf(fid,' The infeasibility of X*  === %9.8e \n', INFOS.infeas);
fprintf(fid,' Final ||Ho(X-G)||        ===== %9.8e \n', INFOS.residue);
fprintf(fid,' Primal function value    ===== %9.8e \n', 0.5*INFOS.residue^2);
fprintf(fid,' Computing time           ======= %.1f(secs) \n', INFOS.time);
fprintf(fid,' ********************************************************* \n');
fclose(fid);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% end of the main program %%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%






%%  **************************************
%%  ******** All Sub-routines  ***********
%%  **************************************

%%% To change the format of time 
function [h,m,s] = time(t)
t = round(t); 
h = floor(t/3600);
m = floor(rem(t,3600)/60);
s = rem(rem(t,60),60);
%%% End of time.m



%%% mexeig decomposition
function [P,lambda] = MYmexeig(X)
[P,lambda] = mexeig(X);
P          = real(P);
lambda     = real(lambda);
if issorted(lambda)
    lambda = lambda(end:-1:1);
    P      = P(:,end:-1:1);
elseif issorted(lambda(end:-1:1))
    return;
else
    [lambda, Inx] = sort(lambda,'descend');
    P = P(:,Inx);
end
return
%%% End of MYmexeig.m



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function X = mPCA(P,lambda,Rank,b)
%lambda>=0 and b>=0
n = length(lambda);
if nargin < 4
    b = ones(n,1);
end

if Rank>0
    P1       = P(:, 1:Rank);
    lambda1  = lambda(1:Rank);
    lambda1  = lambda1.^0.5;
    if Rank>1
        P1 = P1*sparse(diag(lambda1));
    else
        P1 = P1*lambda1;
    end
    pert_Mat = rand(n,Rank);
    for i=1:n
        s = norm(P1(i,:));
        if s<1.0e-12  % PCA breakdowns
            P1(i,:) = pert_Mat(i,:);
            s       = norm(P1(i,:));
        end
        P1(i,:) = P1(i,:)/s;
        P1(i,:) = P1(i,:)*sqrt(b(i));
    end
    X = P1*P1';
else
    X = zeros(n,n);
end
return
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function X  = Projr(P,lambda,r)
n = length(lambda);
X = zeros(n,n);
if r>0
    P1      = P(:,1:r);
    lambda1 = lambda(1:r);
    if r>1
        lambda1 = lambda1.^0.5;
        P1 = P1*sparse(diag(lambda1));
        X  = P1*P1';
    else
        X = lambda1*P1*P1';
    end
end
return
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% use an alternating projection method to generate an initial point
function [X,P,lambda,rank_X,rankErr,infoNum] = ...
    InitialPoint(G,H_max2,ConstrA,Rank,rankErr_CaliMatW,X1)

t0 = clock;

e   = ConstrA.e; I_e = ConstrA.Ie; J_e = ConstrA.Je;
l   = ConstrA.l; I_l = ConstrA.Il; J_l = ConstrA.Jl;
u   = ConstrA.u; I_u = ConstrA.Iu; J_u = ConstrA.Ju;
k_e = length(e); k_l = length(l);  k_u = length(u);
k   = k_e + k_l + k_u;  n = length(G);
Ind = find(I_e==J_e);
if ( length(Ind) == k )
    use_mPCA = 1;
else
    use_mPCA = 0;
end

maxit        = 20;
rank_ratio   = 0.90;
tolinfeas    = 1.0e-6;
c0     = 1.0e1;
cmax   = 1.0e6;
c_step = 2;
c      = c0;

infoNum.callCN      = 0;
infoNum.iterCN      = 0;
infoNum.JacobProd   = 0;
infoNum.eigendecom  = 0;

opts.disp     = 0;        
opts.finalEig = 1;
opts.eps_bar  = 1.0e1;
eps_step      = 1.0e-2;
eps_bar_min   = 1.0e-4;

for iter = 1:maxit
    if iter == 1
        Y = X1;
    else
        if use_mPCA
            Y = mPCA(P,lambda,Rank,e);
        else
            Y = Projr(P,lambda,Rank);
        end
    end
    infeas = zeros(k,1);
    for i=1:k_e
        infeas(i) = e(i) - Y(I_e(i),J_e(i));
    end
    for i=1:k_l
        infeas(k_e+i) = max( l(i) - Y(I_l(i),J_l(i)), 0 );
    end
    for i=1:k_u
        infeas(k_e+k_l+i) = max( Y(I_u(i),J_u(i)) - u(i), 0 );
    end
    NormInf = norm(infeas);
    if ( NormInf <= tolinfeas )
        X = Y;
        [P,lambda] = MYmexeig(X);
        rank_X = Rank;
        rankErr = abs( sum(diag(X)) - sum(lambda(1:Rank)) );        
        infoNum.eigendecom  = infoNum.eigendecom + 1;       
        fprintf('\n Alternating terminates at projection onto rank constraint with good feasibility!')
        if iter == 1
            fprintf('\n *** This initial point is exactly CaliMatWPCA! ***')
        end
        break;
    end
  
    G0 = (H_max2*G + c*Y)/(H_max2 + c);  
    [X,z,info] = CaliMat1Mex_Wnorm(G0,eye(n),ConstrA,opts);   
    P      = info.P;
    lambda = info.lam;
    rank_X = info.rank;
    infoNum.callCN      = infoNum.callCN + 1;
    infoNum.iterCN      = infoNum.iterCN + info.numIter;
    infoNum.JacobProd   = infoNum.JacobProd + info.numJacobProd;
    infoNum.eigendecom  = infoNum.eigendecom + info.numEig;
    rankErr = abs( sum(lambda) - sum(lambda(1:Rank)) );
    if ( rankErr <= rank_ratio*max(1,rankErr_CaliMatW) )
        fprintf('\n Alternating terminates at projection onto linear constraints with small rank error!')
        break;
    end

    %%% update opts.eps_bar and c
    opts.eps_bar = max( eps_step*opts.eps_bar, eps_bar_min );
    c = min( c_step*c, cmax ); 
    
end
fprintf('\n Num of iteration = %2.0d', iter)
fprintf('   Initial RankErr  = %4.3e', rankErr)
time_used = etime(clock, t0);
fprintf('\n Time used to generate the initial point = %.1f',time_used)
return
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





