function [X,INFOS] = PenCorr_lub(G,ConstrA,Rank,OPTIONS)
%%%%%%%%%%%%% This code is designed to solve %%%%%%%%%%%%%%%%%%%%%
%%       min   0.5*|| X-G ||^2
%%       s.t.  X_ii   = e_ij    (i,j) in (I_e,J_e)
%%             X_ij  >= l_ij    (i,j) in (I_l,J_l)
%%             X_ij  <= u_ij    (i,j) in (I_u,J_u)
%%            rank(X) <= Rank     
%%              X    >= tau*I    X is SDP (tau>=0 and may be zero)
%%%
%   Parameters:
%   Input
%   G           the given symmetric matrix
%   ConstrA:    the equality and inequality constraints
%   Rank:       the rank constraint of X
%   OPTIONS:    parameters in the OPTIONS structure
%
%   Output
%   X         the optimal primal solution
%   INFOS     the optimal dual solution to equality constraints
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  Last modified on June 25, 2010.



%%
%%-----------------------------------------
%%% get constraints infos from constrA
%%-----------------------------------------
%%
e   = ConstrA.e; I_e = ConstrA.Ie; J_e = ConstrA.Je;
l   = ConstrA.l; I_l = ConstrA.Il; J_l = ConstrA.Jl;
u   = ConstrA.u; I_u = ConstrA.Iu; J_u = ConstrA.Ju;
k_e = length(e); k_l = length(l);  k_u = length(u);
k   = k_e + k_l + k_u;  n = length(G);

tolrel = 1.0e-5;
%%                                      
%%-----------------------------------------
%% get parameters from the OPTIONS structure. 
%%-----------------------------------------
%%
if exist('OPTIONS')
    if isfield(OPTIONS,'tau');                tau               = OPTIONS.tau; end
    if isfield(OPTIONS,'tolrel');             tolrel            = OPTIONS.tolrel; end
    if isfield(OPTIONS,'tolrank');            tolrank           = OPTIONS.tolrank; end
    if isfield(OPTIONS,'tolsub');             tolsub            = OPTIONS.tolsub; end
    if isfield(OPTIONS,'tolPCA');             tolPCA            = OPTIONS.tolPCA; end
    if isfield(OPTIONS,'tolinfeas');          tolinfeas         = OPTIONS.tolinfeas; end
    if isfield(OPTIONS,'tolsub_rank');        tolsub_rank       = OPTIONS.tolsub_rank; end
    if isfield(OPTIONS,'maxit');              maxit             = OPTIONS.maxit; end
    if isfield(OPTIONS,'maxitsub');           maxitsub          = OPTIONS.maxitsub; end
    if isfield(OPTIONS,'use_CorNewtonPCA');   use_CorNewtonPCA  = OPTIONS.use_CorNewtonPCA ; end
    if isfield(OPTIONS,'use_InitialPoint');   use_InitialPoint  = OPTIONS.use_InitialPoint ; end
end
tau         = 0;
innerTolrel = tolrel;
tolsub      = max(innerTolrel, 1.0*tolrel);  
tolPCA      = max(innerTolrel, 1.0*tolrel);   
tolinfeas   = max(innerTolrel, 1.0*tolrel);   %% the feasibility tolerance 
tolsub_rank = tolsub;  
tolrank     = 1.0e-8;  %%no need to change
maxit    = 500; 
maxitsub = 100;
%use_CorNewtonPCA = 0;  %% set 0 always
use_InitialPoint = 0;  %% =1 initial point from an alternating projection method 
finalPCA         = 0;
residue_cutoff = 10;
if tolrel <= 1.0e-4;
   residue_cutoff = 100;
end

t0 = clock;

%%% reset inpust pars
eye_n  = speye(n);
G      = G - tau*eye_n;   % reset G
G      = (G + G')/2;         % make G symmetric
Ind    = find(I_e==J_e);
e(Ind) = e(Ind) - tau;       % reset the diagonal part of b
e_diag = e(Ind);
%%% constant pars
const.disp1 = 10;
const.disp2 = 10;
const.rank_hist    = 5;  const.rank_hist    = max(2, const.rank_hist);  
const.rankErr_hist = 3;  const.rankErr_hist = max(2, const.rankErr_hist);  
const.funcVal_hist = 2;  const.funcVal_hist = max(2, const.funcVal_hist);
const.residue_hist = const.funcVal_hist;  const.residue_hist = max(2, const.residue_hist); 
rank_hist    = zeros(const.rank_hist,1);
rankErr_hist = zeros(const.rankErr_hist,1);
funcVal_hist = zeros(const.funcVal_hist,1);
residue_hist = zeros(const.residue_hist,1);
progress_rank1    = 1;
progress_rank2    = 1;
%progress_residue  = 1.0e-4;
progress_relErr   = 1.0e-5;
progress_rankErr  = 1.0e-3;

%%% penalty pars
c0_min = 1.0;
c0_max = 1e2;   
alpha_min = 1.2; %1.4
alpha_max = 4.0;
c_max  = 1.0e8;

%%% 
Totalcall_CN    = 0;
Totaliter_CN    = 0;
Totalnumb_JacobProd  = 0;
Totalnumb_eigendecom = 0;

fprintf('\n ******************************************')
fprintf('\n  The problem information: \n')
fprintf(' Dimension of SDP constr.  = %3.0f \n', n)
fprintf(' Fixed upper-diag entries  = %3.0f \n', k_e)
fprintf(' Num. of lower bound       = %3.0f \n', k_l)
fprintf(' Num. of lower bound       = %3.0f \n', k_u)
fprintf(' The required rank of X   <= %3.0f \n', Rank)
%%
%%---------------------------------------------------------------
%%% CaliMat1Mex preprocessing
%%---------------------------------------------------------------
%%
fprintf('\n ^^^^^^^^  Preprocessing by CaliMat1Mex  ^^^^^^^^ ')
opts.disp    = 1;
opts.eps_bar = 1.0e1;
[X,z,info] = CaliMat1Mex(G,ConstrA,opts);
P      = info.P;
lambda = info.lam;
rank_X = info.rank;
Totalcall_CN   = Totalcall_CN + 1;
Totaliter_CN   = Totaliter_CN + info.numIter;
Totalnumb_JacobProd  = Totalnumb_JacobProd + info.numJacobProd;
Totalnumb_eigendecom = Totalnumb_eigendecom + info.numEig;
residue_CorNewton = sum(sum((X-G).*(X-G)));
residue_CorNewton = residue_CorNewton^0.5;
rankErr_CorNewton = abs( sum(lambda) - sum(lambda(1:Rank)) );
if  ( rankErr_CorNewton <= tolrank )  
    fprintf('\n The rank error is already small enough!')
    fprintf('\n The rank error       = %4.3e', rankErr_CorNewton)
    fprintf('\n Residue_CorNewton    = %9.8e', residue_CorNewton)
    time_used = etime(clock,t0);
    fprintf('\n Total computing time = %.1f(secs) \n', time_used); 
    INFOS.iter        = 0;
    INFOS.callCN      = Totalcall_CN;
    INFOS.itCN        = Totaliter_CN;
    INFOS.itJacobProd = Totalnumb_JacobProd;
    INFOS.numEig      = Totalnumb_eigendecom;
    INFOS.rank    = rank_X;
    INFOS.rankErr = rankErr_CorNewton;
    INFOS.residue = residue_CorNewton;
    INFOS.time    = time_used;
    return;
end
fprintf('\n The rank of NCM        = %2.0d', rank_X)
fprintf('\n The rank error of NCM  = %4.3e', rankErr_CorNewton)  
fprintf('\n Residue_CorNewton      = %9.8e', residue_CorNewton)


%%
%%---------------------------------------------------------------
%%% CorNewton_PCA
%%---------------------------------------------------------------
%%
X1 = mPCA(P,lambda,Rank,e_diag);
residue_CorNewtonPCA = sum(sum((X1-G).*(X1-G)));
residue_CorNewtonPCA = residue_CorNewtonPCA^0.5; 
residue_error = abs( residue_CorNewtonPCA - residue_CorNewton );
fprintf('\n Residue_CorNewtonPCA  = %9.8e',residue_CorNewtonPCA)
infeas = zeros(k,1);
for i=1:k_e
    infeas(i) = e(i) - X1(I_e(i),J_e(i));
end
for i=1:k_l
    infeas(k_e+i) = max( l(i)-X1(I_l(i),J_l(i)), 0 );
end
for i=1:k_u
    infeas(k_e+k_l+i) = max( X1(I_u(i),J_u(i)) - u(i), 0 );
end
NormInf_CorNewtonPCA = norm(infeas);   
if ( residue_error/max(residue_cutoff, residue_CorNewtonPCA ) <= tolPCA && NormInf_CorNewtonPCA <= tolinfeas )
    fprintf('\n CorNewton_PCA is good enough!')
    time_used = etime(clock,t0);
    fprintf('\n Total computing time   = %.1f', time_used);
    INFOS.iter    = 0;
    INFOS.callCN  = Totalcall_CN;
    INFOS.itCN    = Totaliter_CN;
    INFOS.itJacobProd  = Totalnumb_JacobProd;
    INFOS.numEig       = Totalnumb_eigendecom;
    INFOS.rank    = Rank;
    INFOS.rankErr = 0;
    INFOS.residue = residue_CorNewtonPCA;
    INFOS.time    = time_used;
    return;
end

%%
%%---------------------------------------------------------------
%%% Initial point
%%---------------------------------------------------------------
%%
fprintf('\n\n ^^^^^^^^ Initial Guess ^^^^^^^^ ')
if use_InitialPoint  %% use alternating projection method to generate initial point
   fprintf('\n use InitialPoint!') 
   opt_disp = 0;
   [X,P,lambda,rank_X,rankErr,normInf,infoNum] = IntPoint_lub(G,ConstrA,Rank,X,P,lambda,opt_disp);
   
   %[X,P,lambda,rank_X,rankErr,infoNum] = InitialPoint(G,ConstrA,Rank,rankErr_CorNewton,X1);   
    Totalcall_CN    = Totalcall_CN + infoNum.callCN;
    Totaliter_CN    = Totaliter_CN + infoNum.iterCN;
    Totalnumb_JacobProd  = Totalnumb_JacobProd + infoNum.JacobProd;
    Totalnumb_eigendecom = Totalnumb_eigendecom + infoNum.eigendecom;
    residue_int = sum(sum((X-G).*(X-G)));
    residue_int = residue_int^0.5;
    residue_1   = residue_int;
else %%use_CorNewtonPCA 
    fprintf('\n use CorNewton_PCA!') 
    X = X1;
    [P,lambda] = MYmexeig(X);
    Totalnumb_eigendecom = Totalnumb_eigendecom + 1;
    rank_X      = Rank;
    residue_int = residue_CorNewtonPCA;  
    residue_1   = residue_int;
end

%%% initialize U
P1 = P(:,1:Rank);
U  = P1*P1';
rankErr = abs( sum(lambda) - sum(lambda(1:Rank)) );

%%
%%---------------------------------------------------------------
%%% initial penalty parameter c
%%---------------------------------------------------------------
%%
if use_InitialPoint    
    c0 = 0.50*(residue_int^2 - residue_CorNewton^2);
    c0 = 0.25*c0/max(1.0, rankErr_CorNewton - rankErr);
else  %use_CorNewtonPCA
    c0 = 0.50*(residue_CorNewtonPCA^2 - residue_CorNewton^2);
    c0 = 0.25*c0/max(1.0, rankErr_CorNewton);    
end
if Rank <= 1
    c0 = c0_max;
end

if tolrel >= 1.0e-1;  %% less acurate, larger c
    c0 = 4*c0;
elseif tolrel >= 1.0e-2;  %% less acurate, larger c
    c0 = 2*c0;
% elseif tolrel >= 1.0e-3;  %% less acurate, larger c
%     c0 = 2*c0;
% elseif tolrel >= 1.0e-4;  %% less acurate, larger c
%     c0 = 2*c0;
end
% if tolrel <= 1.0e-6;  %% more acurate, smaller c
%     c0 = c0/2;
% end
c0 = max(c0, c0_min);
c0 = min(c0, c0_max);
c  = c0;


fprintf('\n\n ************************************** \n')
fprintf( '      The Penalty Method Initiated!!!      ')
fprintf('\n ************************************** \n')
fprintf('The initial rank        = %3.0d \n', rank_X);
fprintf('The initial rank error  = %4.3e \n', rankErr);
fprintf('The initial ||X0-G||    = %5.4e \n', residue_1);

relErr_0    = 1.0e6;
break_level = 0;
eps_bar_min  = 1.0e-4;
opts.eps_bar = 1.0e1;

k1        = 1;
sum_iter  = 0;
opts.disp = 0;  
while ( k1 <= maxit )    
    subtotaliter_CN    = 0;
    subtotalnumb_JacobProd  = 0;
    subtotalnumb_eigendecom = 0;
        
    fc = 0.5*residue_1^2;
    fc = fc + c*rankErr;
           
    tt = etime(clock,t0);
    fprintf('\n ============')
    fprintf(' The %2.0dth level of penalty par. %3.2e',k1,c)
    fprintf('  ====================')
    fprintf('\n ........Calling CaliMat1Mex')
    fprintf('\n CallNo.  NumIt  JacobProd    RankX     RankErr      Sqrt(2*FunVal)     Time')
    fprintf('\n %2.0fth      %s       %s          %3.0d    %5.4e    %9.8e    %.1f',...
        0,'-','-',rank_X,rankErr,sqrt(2)*fc^0.5,tt)   
 
    G0 = G + c*(U - eye(n)); 
    for itersub = 1:maxitsub       
        
        if ( itersub == 1 && rankErr > tolrank )
            [X,z,info] = CaliMat1Mex(G0,ConstrA,opts);
        else
            [X,z,info] = CaliMat1Mex(G0,ConstrA,opts,z);
        end
        P          = info.P;
        lambda     = info.lam;
        rank_X     = info.rank;
        major_dualVal = info.dualVal;
        Totalcall_CN             = Totalcall_CN + 1;
        subtotalnumb_JacobProd   = subtotalnumb_JacobProd + info.numJacobProd;
        subtotaliter_CN          = subtotaliter_CN + info.numIter;
        subtotalnumb_eigendecom  = subtotalnumb_eigendecom + info.numEig;        
        rankErr    = abs( sum(lambda) - sum(lambda(1:Rank)) ); 
        fc         = sum(sum((X-G).*(X-G)));
        residue_1  = fc^0.5;        
        fc         = 0.5*fc + c*rankErr;
                                            
        if ( itersub <= const.disp1 || mod(itersub,const.disp2) == 0 )
            dispsub = 1;
            tt = etime(clock,t0);
            fprintf('\n %2.0dth     %2.0f      %3.0f         %3.0d    %5.4e    %9.8e    %.1f',...
                itersub, info.numIter, info.numJacobProd, rank_X, rankErr, sqrt(2)*fc^0.5, tt)
            
            %fprintf('\n The primal value of the majorized problem = %5.4e',fc)
            %fprintf('\n The dual value of the majorized problem   = %5.4e',major_dualVal)  %%% added on Jan 29, 2010.
            
        else
            dispsub = 0;
        end
     
        %%% rank history
        if  itersub <= const.rank_hist
            rank_hist(itersub) = rank_X;
        else
            for j = 1:const.rank_hist-1
                rank_hist(j) = rank_hist(j+1);
            end
            rank_hist(const.rank_hist) = rank_X;
        end
        %%% function value history
        if  itersub <= const.funcVal_hist
            funcVal_hist(itersub) = fc^0.5;
        else
            for j = 1:const.funcVal_hist-1
                funcVal_hist(j) = funcVal_hist(j+1);
            end
            funcVal_hist(const.funcVal_hist) = fc^0.5;
        end
        %%% residue history
        if sum_iter + itersub <= const.residue_hist
            residue_hist(sum_iter + itersub) = residue_1;
        else
            for j = 1:const.residue_hist-1
                residue_hist(j) = residue_hist(j+1);
            end
            residue_hist(const.residue_hist) = residue_1;
        end
        
        if rankErr <= tolrank
            tolsub_check = tolsub_rank;       
        else
            tolsub_check = tolsub*max(10, min(100,rank_X/Rank));
        end
        if itersub >= const.funcVal_hist
            relErr_sub  = abs(funcVal_hist(1) - funcVal_hist(const.funcVal_hist));
            relErr_sub  = relErr_sub/max(residue_cutoff, max(funcVal_hist(1), funcVal_hist(const.funcVal_hist)));
        end
        if ( itersub >= const.funcVal_hist &&  relErr_sub <= tolsub_check )
            if dispsub == 0
                tt = etime(clock,t0);
                fprintf('\n %2.0dth     %2.0f      %3.0f         %3.0d    %5.4e    %9.8e    %.1f',...
                    itersub, info.numIter, info.numJacobProd, rank_X, rankErr, sqrt(2)*fc^0.5,tt)
                
                %fprintf('\n The primal value of the majorized problem = %5.4e',fc)
                %fprintf('\n The dual value of the majorized problem   = %5.4e',major_dualVal)  %%% added on Jan 29, 2010.
                
            end
            break;
        elseif  ( itersub >= const.rank_hist && abs( rank_hist(1) - rank_hist(const.rank_hist) ) <= progress_rank1...
                && rank_X - Rank >= progress_rank2 )
            if dispsub == 0
                tt = etime(clock,t0);
                fprintf('\n %2.0dth     %2.0f      %3.0f       %3.0d      %5.4e    %9.8e    %.1f',...
                    itersub, info.numIter, info.numJacobProd, rank_X,rankErr, sqrt(2)*fc^0.5,tt)
                
                %fprintf('\n The primal value of the majorized problem = %5.4e',fc)
                %fprintf('\n The dual value of the majorized problem   = %5.4e',major_dualVal)  %%% added on Jan 29, 2010.
                
                
            end
            break;
        end
        
        % update U, G0 and fc0
        P1  = P(:, 1:Rank);
        U   = P1*P1';
        G0  = G + c*(U - eye(n));
        
        if itersub <= const.residue_hist
            opts.eps_bar = opts.eps_bar/(10^itersub);
            opts.eps_bar = max( opts.eps_bar, 1.0e-6 );
        else
            opts.eps_bar = min(opts.eps_bar, min(1,relErr_sub)^2);
            opts.eps_bar = max( opts.eps_bar, 1.0e-6 );
        end
       
    end   % end of subproblem
    
    sum_iter        = sum_iter + itersub;  
    Totaliter_CN    = Totaliter_CN + subtotaliter_CN;
    Totalnumb_JacobProd  = Totalnumb_JacobProd + subtotalnumb_JacobProd;    
    Totalnumb_eigendecom = Totalnumb_eigendecom + subtotalnumb_eigendecom;
    fprintf('\n SubTotal %2.0f      %2.0f      %2.0f(eigendecom)',...
        subtotaliter_CN,subtotalnumb_JacobProd,subtotalnumb_eigendecom);
   
    if sum_iter >= const.residue_hist
        relErr = abs(residue_hist(1) - residue_hist(const.residue_hist));
        relErr = relErr/max(residue_cutoff, max(residue_hist(1), residue_hist(const.residue_hist)));
    else
        relErr = abs(residue_hist(1) - residue_hist(sum_iter));
        relErr = relErr/max(residue_cutoff, max(residue_hist(1), residue_hist(sum_iter)));
    end
    tt = etime(clock,t0);
    [hh,mm,ss] = time(tt);
    fprintf('\n   Iter.   PenPar.    Rank(X)    RankError     RelErr     ||X-G||      Time_used ')
    fprintf('\n   %2.0d     %3.2e    %3.0d      %5.4e   %3.2e  %5.4e     %d:%d:%d \n',...
        k1,c,rank_X,rankErr,relErr,residue_1,hh,mm,ss)

    if  k1 <= const.rankErr_hist
        rankErr_hist(k1) = rankErr;
    else
        for j=1:const.rankErr_hist-1
            rankErr_hist(j) = rankErr_hist(j+1);
        end
        rankErr_hist(const.rankErr_hist) = rankErr;
    end    
    %%% termination test
    if ( relErr <= tolrel )
        if ( rankErr <= tolrank )
            fprintf('\n The rank constraint is satisfied!')
            break;
        elseif ( k1 >= const.rankErr_hist && abs(rankErr_hist(1) - rankErr_hist(const.rankErr_hist)) <= progress_rankErr )
            fprintf('\n Warning: The rank does not decrease any more! :( ')
            finalPCA = 1;
            break;
        end
    else
        if ( abs(relErr_0 - relErr)/max(1,relErr) <= progress_relErr )
            break_level = break_level + 1;
            if break_level == 3
                fprintf('\n Warning: The relErr is consecutively decreasing slowly, quit! :( ')
                if ( rankErr > tolrank )
                    finalPCA = 1;
                end
                break;
            end
        end        
    end
    
    k1        = k1 + 1;
    relErr_0  = relErr;
    
    %%% update c
    if rank_X <= Rank
        c = min(c_max, c);
        fprintf('\n The rank constraint is satisfied and keep c the same!')
    else
        if rankErr/max(1,Rank) > 1.0e-1
            c = min(c_max, c*alpha_max);
        else
            c = min(c_max, c*alpha_min);
        end
    end
    %%% update eps_bar for next level penalty
    if rank_X <= Rank
        opts.eps_bar = max( opts.eps_bar, eps_bar_min );
    else
        if k1 <= 3         %const.residue_hist
            opts.eps_bar = opts.eps_bar/(10^k1);
            opts.eps_bar = max( opts.eps_bar, eps_bar_min );
        else
            opts.eps_bar = min(opts.eps_bar, min(1,relErr)^2);
            opts.eps_bar = max( opts.eps_bar, eps_bar_min );
        end
    end
    
end

%%% final PCA correction
if finalPCA 
        X = mPCA(P,lambda,Rank,e_diag);
        %[P,lambda] = MYmexeig(X);
        %Totalnumb_eigendecom = Totalnumb_eigendecom + 1;
        rank_X    = Rank;
        rankErr   = 0;
        residue_1 = sum(sum((X-G).*(X-G)))^0.5; 
end
infeas = zeros(k,1);
for i=1:k_e
    infeas(i) = e(i) - X(I_e(i),J_e(i));
end
for i=1:k_l
    infeas(k_e+i) = max( l(i) - X(I_l(i),J_l(i)), 0 );
end
for i=1:k_u
    infeas(k_e+k_l+i) = max( X(I_u(i),J_u(i)) - u(i), 0 );
end
NormInf = norm(infeas);

time_used = etime(clock,t0);
INFOS.iter      = k1;
INFOS.callCN    = Totalcall_CN;
INFOS.itCN      = Totaliter_CN;
INFOS.itJacobProd = Totalnumb_JacobProd;
INFOS.numEig      = Totalnumb_eigendecom;
INFOS.rank      = rank_X;
INFOS.rankErr   = rankErr;
INFOS.relErr    = relErr;
INFOS.infeas    = NormInf;
INFOS.residue   = residue_1;
INFOS.time      = time_used;
fprintf('\n Final ||X-G||            === %9.8e', INFOS.residue);
fprintf('\n Primal function value    === %9.8e', 0.5*INFOS.residue^2);
fprintf('\n MajorDual function value === %9.8e', major_dualVal);
fprintf('\n Computing time           ======= %.1f(secs) \n', INFOS.time);

fid = fopen('result_PenCorr_lub.txt','a+');
fprintf(fid,'\n ****** result_PenCorr_lub ******************');
fprintf(fid,'\n  The problem information: \n');
fprintf(fid,' Dim.  of    sdp        constr  = %d \n',n);
fprintf(fid,' Num. of fixed elements         = %d \n',k_e);
fprintf(fid,' Num. of lower bound            = %d \n',k_l);
fprintf(fid,' Num. of lower bound            = %d \n',k_u);
fprintf(fid,' The required rank      constr  = %d \n',Rank);
fprintf(fid,' ---------------------------------------------------- \n');
%fprintf(fid,'\n **************** Final Information ******************** \n');
fprintf(fid,' Num of pen level         = %d \n', INFOS.iter);
fprintf(fid,' Num of calling CN        = %d \n', INFOS.callCN);
fprintf(fid,' Total num of iter in CN  = %d \n', INFOS.itCN);
fprintf(fid,' Total num of BiCG(CG)    = %d \n', INFOS.itJacobProd);
fprintf(fid,' Total num of eigendecom  = %d \n', INFOS.numEig);
fprintf(fid,' The rank of  X*          === %d \n', INFOS.rank);
fprintf(fid,' The rank error           === %3.2e \n', INFOS.rankErr);
fprintf(fid,' The rel func error       ===== %3.2e \n', INFOS.relErr);
fprintf(fid,' Final ||X*-G||           ===== %9.8e \n', INFOS.residue);
fprintf(fid,' Computing time           ======= %.1f(secs) \n', INFOS.time);
fprintf(fid,' ********************************************************* \n');
fclose(fid);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% end of the main program %%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%






%%  **************************************
%%  ******** All Sub-routines  ***********
%%  **************************************
%%% To change the format of time 
function [h,m,s] = time(t)
t = round(t); 
h = floor(t/3600);
m = floor(rem(t,3600)/60);
s = rem(rem(t,60),60);
%%% End of time.m



%%% mexeig decomposition
function [P,lambda] = MYmexeig(X)
[P,lambda] = mexeig(X);
P          = real(P);
lambda     = real(lambda);
if issorted(lambda)
    lambda = lambda(end:-1:1);
    P      = P(:,end:-1:1);
elseif issorted(lambda(end:-1:1))
    return;
else
    [lambda, Inx] = sort(lambda,'descend');
    P = P(:,Inx);
end
return
%%% End of MYmexeig.m



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function X = mPCA(P,lambda,Rank,b)
%lambda>=0 and b>=0
n = length(lambda);
if nargin < 4
    b = ones(n,1);
end

if Rank>0
    P1       = P(:, 1:Rank);
    lambda1  = lambda(1:Rank);
    lambda1  = lambda1.^0.5;
    if Rank>1
        P1 = P1*sparse(diag(lambda1));
    else
        P1 = P1*lambda1;
    end
    pert_Mat = rand(n,Rank);
    for i=1:n
        s = norm(P1(i,:));
        if s<1.0e-12  % PCA breakdowns
            P1(i,:) = pert_Mat(i,:);
            s       = norm(P1(i,:));
        end
        P1(i,:) = P1(i,:)/s;
        P1(i,:) = P1(i,:)*sqrt(b(i));
    end
    X = P1*P1';
else
    X = zeros(n,n);
end
return
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function X  = Projr(P,lambda,r)
n = length(lambda);
X = zeros(n,n);
if r>0
    P1      = P(:,1:r);
    lambda1 = lambda(1:r);
    if r>1
        lambda1 = lambda1.^0.5;
        P1 = P1*sparse(diag(lambda1));
        X  = P1*P1';
    else
        X = lambda1*P1*P1';
    end
end
return
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% use an alternating projection method to generate an initial point
function [X,P,lambda,rank_X,rankErr,infoNum] = ...
    InitialPoint(G,ConstrA,Rank,rankErr_CorNewton,X1)

t0 = clock;

e   = ConstrA.e; I_e = ConstrA.Ie; J_e = ConstrA.Je;
l   = ConstrA.l; I_l = ConstrA.Il; J_l = ConstrA.Jl;
u   = ConstrA.u; I_u = ConstrA.Iu; J_u = ConstrA.Ju;
k_e = length(e); k_l = length(l);  k_u = length(u);
k   = k_e + k_l + k_u;  
Ind = find(I_e==J_e);
if ( length(Ind) == k )
    use_mPCA = 1;
else
    use_mPCA = 0;
end

maxit      = 20;
rank_ratio = 0.90;
tolinfeas  = 1.0e-6;
c0    = 1.0e1;
cmax  = 1.0e6;
c_step = 2;
c      = c0;

infoNum.callCN      = 0;
infoNum.iterCN      = 0;
infoNum.JacobProd   = 0;
infoNum.eigendecom  = 0;

opts.disp    = 0;
opts.eps_bar = 1.0e1;
eps_step     = 1.0e-2;
eps_bar_min  = 1.0e-4;
for iter = 1:maxit
    if iter == 1
        Y = X1;
    else
        if use_mPCA
            Y = mPCA(P,lambda,Rank,e);
        else
            Y = Projr(P,lambda,Rank);
        end
    end
    infeas = zeros(k,1);
    for i=1:k_e
        infeas(i) = e(i) - Y(I_e(i),J_e(i));
    end
    for i=1:k_l
        infeas(k_e+i) = max( l(i) - Y(I_l(i),J_l(i)), 0 );
    end
    for i=1:k_u
        infeas(k_e+k_l+i) = max( Y(I_u(i),J_u(i)) - u(i), 0 );
    end
    NormInf = norm(infeas);
    if ( NormInf <= tolinfeas )
        X = Y;
        [P,lambda] = MYmexeig(X);
        rank_X     = Rank;
        rankErr    = abs( sum(diag(X)) - sum(lambda(1:Rank)) );
        infoNum.eigendecom  = infoNum.eigendecom + 1;
        fprintf('\n Alternating terminates at projection onto rank constraint with good feasibility!')
        if iter == 1
            fprintf('\n *** This initial point is exactly CorNewtonPCA! ***')
        end
        break;
    end
    
    G0 = (G + c*Y)/(1+c);                
    [X,z,info] = CaliMat1Mex(G0,ConstrA,opts);
    P       = info.P;
    lambda  = info.lam;
    rank_X  = info.rank;
    rankErr = abs( sum(diag(X)) - sum(lambda(1:Rank)) );
    infoNum.callCN      = infoNum.callCN + 1;
    infoNum.iterCN      = infoNum.iterCN + info.numIter;
    infoNum.JacobProd   = infoNum.JacobProd + info.numJacobProd;
    infoNum.eigendecom  = infoNum.eigendecom + info.numEig;    
    if ( rankErr <= rank_ratio*max(1,rankErr_CorNewton) )
        fprintf('\n Alternating terminates at projection onto linear constraints with smaller rank error!')
        break;
    end
    
    %%% update opts.eps_bar and c
    opts.eps_bar = max( eps_step*opts.eps_bar, eps_bar_min );
    c = min( c_step*c, cmax );       
end
fprintf('\n Num of iteration = %2.0f',iter)
fprintf('   RankErr          = %4.3e',rankErr)
time_used = etime(clock, t0);
fprintf('\n Time used to generate the initial point = %.1f',time_used)
return
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





