function [Xn, Fn] = majorw_source( R, d, WW, ftol, ftest, gradtol , tolrel)
 
%MAJOR Majorisation algorithm for rank reduction of correlation matrices
%  Majorization algorithm for minimization of F(X)=\sum_{i<j}w_{ij}(r_{ij}-<x_i,x_j>)^2 such that diag(X*X')=I with X an nxd matrix with n=size(R,1)=size(R,2). 
%  Convergence is guaranteed if the matrix R has unit diagonal and if all elements of W are positive. 
%  gradtol is the convergence criterion for the norm of the gradient 4*Psi*X with Psi = X*X'-R.
%  ftol is the convergence criterion for the improvement in the function
%  value F. Both criteria should be satisfied for convergence.
%  usage [Xn, Fn] = majorw( R, d, W, ftol, gradtol )
t0 = clock;


residue_cutoff = 10;
if tolrel <= 1.0e-4;
    residue_cutoff = 100;
end
     
const1_disp = 10;
const2_disp = 10;
disp_level = 1;


const_hist = 20000;
hist_residue = zeros(const_hist,1);
n_base = 10;


    % first obtain initial guess
    X=guess(R,d);
    % dimension
    N=size(R,1);

WeightFlag = any(any(1-WW));  %% =1 nonequal weight on Oct.6, 2009.

if WeightFlag
    W = WW.^2;   %%% on Nov 3, 2009
else
    W = WW;
end

%%% sum of weights
if WeightFlag
    sumW = 0.0;
    for i=1:N
        for j=(i+1):N
            sumW = sumW + W(i,j);
        end
    end
else
    sumW = N*(N-1)/2;
end
    
    % initial gradient, obj value
    mygrad=P_tangent( X, dFw( X, R, W, sumW ) );
    normgrad=norm(mygrad,'fro');
    Fn=Fw(X,R,W,sumW);
   % Fold=Fn*(1.0+ftol)+1.0; % at first iteration no Fold is known, this choice guarantees that convergence is not obtained at the first iteration
    residue_1 = sqrt(8*sumW*Fn);
    hist_residue(1) =  residue_1;
    
    
   % output
iter = 0;
time = etime(clock,t0);
fprintf('\n')
disp(sprintf('%s\t%s\t\t%s\t\t\t%s\t%s','iter','|grad|','F(X)','Time'));
disp(sprintf('%d\t%3.2e\t%9.8e\t\t%.1f\t\t%.1f',iter,normgrad,sqrt(8*sumW*Fn),time));


% useful in later calculations
if WeightFlag
    %W_bar = WW;   % W_bar = W.^0.5;    
    WR    = W.*R;
else
    WR = R;
end

    while (normgrad>gradtol) || ( Fold/Fn-1.0 > ftol)
                
        for i=1:N

            % calculate B and its largest eigenvalue lambda
            B=zeros(d);
            for j=1:(i-1)
                B=B+W(i,j)*(X(j,:)'*X(j,:));
            end
            for j=(i+1):N
                B=B+W(i,j)*(X(j,:)'*X(j,:));
            end
            lambda= max(eig(B));
            
            % calculate the position at which the minimum of the
            % majorization function is obtained
            z=(lambda-WR(i,i))*X(i,:)-(B*X(i,:)')'+WR(i,:)*X;
            X(i,:)=z/norm(z);
        
        end
        
        % gradient, obj value
        iter = iter + 1;
        mygrad=P_tangent( X, dFw( X, R, W, sumW ) );
        normgrad=norm(mygrad,'fro');
        Fold=Fn;
         
        Fn = Fw(X,R,W,sumW);
        residue_1 = sqrt(8*sumW*Fn);
        hist_residue(iter+1) = residue_1;
        
        % output
        if ( iter <= const1_disp || mod(iter,const2_disp) == 0 )
            disp_level = 1;
            time = etime(clock,t0);
            disp(sprintf('%d\t%3.2e\t%9.8e\t\t%.1f\t\t%.1f',iter,normgrad,sqrt(8*sumW*Fn),time));
        else
            disp_level = 0;
        end
        
        %%% termination test
        if  iter >= n_base-1
            relErr = abs( hist_residue(iter+2-n_base) - hist_residue(iter+1))...
                / max( residue_cutoff , max(hist_residue(iter+1), hist_residue(iter+2-n_base)));
            if relErr <= tolrel
                if disp_level == 0
                    disp(sprintf('%d\t%3.2e\t%9.8e\t\t%.1f\t\t%.1f',iter,normgrad,sqrt(8*sumW*Fn),time));
                    disp_level = 1;
                end
                fprintf('relative error = %8.7e \n',relErr)
                break;
            end
        end
        
    end
    
Xn=X;





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function nY = clamp(Y)
% CLAMP		rescales the n row vectors of the matrix Y to unit length. If a row vector a priori has length zero, a
% random choice is made.
%
%	Usages:
%	nY = CLAMP(Y)
%
%	Y is expected to satisfy diag(Y*Y')=I to order epsilon
%	nY will satisfy Y'*Y = I 
%

    [n,d]=size(Y);    
    
    % set correct dimensions
    nY=Y;
        
    for i=1:n
        normY=norm( Y(i,:), 'fro' );
        if normY >0.0
            nY(i,:) = Y(i,:) / normY;
        else
            nY(i,:) = zeros(1,d); nY(i,1)=1.0;
        end
    end
    
    
    

function result = dFw( X, R, W, sumW )
% result = dFw( X, R, W )
% the differential of Fw
    
    [n,d]=size(X);
    result = zeros(n,d);
    XXT=X*X';
    for k=1:n
        for l=1:d
            for j=1:(k-1)
                result(k,l)=result(k,l)+W(k,j)*(R(k,j)-XXT(k,j))*X(j,l);
            end
            for j=(k+1):n
                result(k,l)=result(k,l)+W(k,j)*(R(k,j)-XXT(k,j))*X(j,l);
            end
        end
    end
    result = -2.0 * result / (4.0 * sumW);
    
    
    
    function [ f ] = Fw( X, R, W, sumW )
%F F(X)=\sum_{i<j}w_{ij}(r_{ij}-<x_i,x_j>)^2
%  F(X)=\sum_{i<j}w_{ij}(r_{ij}-<x_i,x_j>)^2

XXT=X*X';
n=size(R,1);
f=0.0;
for i=1:n
    for j=(i+1):n
        diff=R(i,j)- XXT(i,j);
        f=f+W(i,j)*diff^2;
    end
end
f=f/(4.0*sumW);



function R = RebonatoForm( n, dt, beta, LongCorr )
% returns an nxn matrix given by
% R_ij = LongCorr + (1-LongCorr)exp(-beta|T_i-T_j|), i,j=1,...,n
% with T_i = i*dt
% USAGE: R = RebonatoForm( n, dt, beta, LongCorr )

R=zeros(n);
for i=1:n
    for j=1:n
        R(i,j)=LongCorr+(1-LongCorr)*exp(-beta*abs(i-j)*dt);
    end
end


    
    function Delta = P_tangent( Y, X )
% Delta = P_tangent( Y, X )
% projects X onto the tangent space at Y thereby producing Delta
% The manifold here is { Y nxd real matrix, norm(Y(i,:))=1 i=1:n } (n products of the d-1 sphere)

Delta = X - diag( diag( X * Y' ) ) * Y;



function Y = guess( C, d, varargin )
%guess Y=guess(C,d)
% usages:
%   Y=guess(C,d)
%   Y=guess(C,d,W)
% Finds an initial feasible point Y to the problem min norm(R-X*X','fro')^2 subject to diag(X*X')=I with X an nxd
% matrix, n=size(C,1).
% If W is provided the factorisation is performed with respect to the matrix norm induced by the positive semidefinite matrix W
% the 'default' for W is I

    % dim
    n=size(C,1);
        
    % is a weights matrix provided?
    if length( varargin ) == 1
        W=varargin{1};
    else
        W=eye(n);
    end
    
    % square root of W
    sqrW = W^0.5;
    isqrW = inv(sqrW);

    % project onto rank d
    [V,L] = svdplus( sqrW*C*sqrW );
    L=max( L, zeros(n) );
    Y = isqrW*V(:,1:d)*sqrt(L(1:d,1:d));
        
    % rescale Y
    Y = clamp( Y );

    
    
    function [V,L]=svdplus(A)  % A must be symmetric !!! 
[n,n]=size(A);
[Q,D]=eig(A);
[B,Index]= sort(diag(-real(D)));         % because the ordening a priori is ascending 
L=diag(-B);
for i=1:n
    V(:,i)=Q(:,Index(i));
end